@@ -10,6 +10,7 @@
 #![feature(const_loop)]
 #![feature(const_mut_refs)]
 #![feature(const_panic)]
+#![feature(const_ptr_offset_from)]
 #![feature(const_raw_ptr_deref)]
 #![feature(const_raw_ptr_to_usize_cast)]
 #![feature(core_intrinsics)]
@@ -26,12 +27,15 @@
 
 pub use crate::errors::{CapacityError, PushCapacityError};
 pub use crate::iterators::*;
-pub use crate::trait_impls::*;
+pub use crate::string::string_utils;
+pub use crate::string::{StaticString, StringError};
 use crate::utils::{
   quicksort_internal, reverse_copy, slice_from_raw_parts, slice_from_raw_parts_mut,
 };
 use core::cmp::{Ord, PartialEq};
 use core::intrinsics;
+#[doc(hidden)]
+pub use core::iter::FromIterator;
 use core::marker::PhantomData;
 use core::mem::MaybeUninit;
 use core::ops::{
@@ -56,6 +60,8 @@ mod iterators;
 mod macros;
 #[doc(hidden)]
 mod errors;
+#[doc(hidden)]
+mod string;
 mod trait_impls;
 #[doc(hidden)]
 pub mod utils;
@@ -119,6 +125,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   ///
   /// Example usage:
   /// ```
+  /// # use staticvec::StaticVec;
   /// // Same input length as the declared capacity:
   /// let v = StaticVec::<i32, 3>::new_from_array([1, 2, 3]);
   /// assert_eq!(v, [1, 2, 3]);
@@ -130,6 +137,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// and static arrays, which may prove more ergonomic in some cases as it allows
   /// for a greater degree of type inference:
   /// ```
+  /// # use staticvec::StaticVec;
   /// // The StaticVec on the next line is inferred to be of type `StaticVec<&'static str, 4>`.
   /// let v = StaticVec::from(["A", "B", "C", "D"]);
   /// ```
@@ -271,14 +279,14 @@ impl<T, const N: usize> StaticVec<T, N> {
   #[inline(always)]
   pub const fn as_ptr(&self) -> *const T {
     // Written like this so it can be `const fn`.
-    &self.data as *const _ as *const T
+    &self.data as *const MaybeUninit<[T; N]> as *const T
   }
 
   /// Returns a mutable pointer to the first element of the StaticVec's internal array.
   #[inline(always)]
   pub const fn as_mut_ptr(&mut self) -> *mut T {
     // Written like this so it can be `const fn`.
-    &mut self.data as *mut _ as *mut T
+    &mut self.data as *mut MaybeUninit<[T; N]> as *mut T
   }
 
   /// Returns a constant reference to a slice of the StaticVec's inhabited area.
@@ -741,6 +749,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   ///
   /// Example usage:
   /// ```
+  /// # use staticvec::{staticvec, StaticVec};
   /// const V: StaticVec<StaticVec<i32, 2>, 2> = staticvec![staticvec![1, 3], staticvec![4, 2]];
   /// assert_eq!(
   ///   V.iter().flatten().collect::<StaticVec<i32, 4>>().sorted(),
@@ -801,10 +810,7 @@ impl<T, const N: usize> StaticVec<T, N> {
         .copy_to_nonoverlapping(Self::first_ptr_mut(&mut res), length);
     }
     quicksort_internal(Self::first_ptr_mut(&mut res), 0, (length - 1) as isize);
-    Self {
-      data: res,
-      length: length,
-    }
+    Self { data: res, length }
   }
 
   /// Returns a separate, reversed StaticVec of the contents of the StaticVec's
@@ -824,6 +830,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   ///
   /// Example usage:
   /// ```
+  /// # use staticvec::StaticVec;
   /// let mut i = 0;
   /// let v = StaticVec::<i32, 64>::filled_with(|| { i += 1; i });
   /// assert_eq!(v.len(), 64);
@@ -854,6 +861,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   ///
   /// Example usage:
   /// ```
+  /// # use staticvec::StaticVec;
   /// let v = StaticVec::<usize, 64>::filled_with_by_index(|i| { i + 1 });
   /// assert_eq!(v.len(), 64);
   /// assert_eq!(v[0], 1);
@@ -950,6 +958,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   ///
   /// Example usage:
   /// ```
+  /// # use staticvec::staticvec;
   /// assert_eq!(
   ///  staticvec!["A, B"].concat(&staticvec!["C", "D", "E", "F"]),
   ///  ["A, B", "C", "D", "E", "F"]
@@ -1006,9 +1015,10 @@ impl<T, const N: usize> StaticVec<T, N> {
   ///
   /// Example usage:
   /// ```
+  /// # use staticvec::staticvec;
   /// assert_eq!(
   ///  staticvec!["A", "B", "C", "D"].intersperse("Z"),
-  ///  ["A, "Z", B", "Z", "C", "Z", "D"]
+  ///  ["A", "Z", "B", "Z", "C", "Z", "D"]
   /// );
   /// ```
   #[inline]
@@ -1251,7 +1261,7 @@ impl<T, const N: usize> StaticVec<T, N> {
 
   /// Splits the StaticVec into two at the given index.
   /// The original StaticVec will contain elements `0..at`,
-  /// and the new one will contain elements `at..length`.
+  /// and the new one will contain elements `at..self.len()`.
   #[inline]
   pub fn split_off(&mut self, at: usize) -> Self {
     let length = self.length;
@@ -1312,6 +1322,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   ///
   /// Example usage:
   /// ```
+  /// # use staticvec::staticvec;
   /// assert_eq!(
   ///   staticvec![4, 5, 6, 7].difference(&staticvec![1, 2, 3, 7]),
   ///   [4, 5, 6]
@@ -1348,6 +1359,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   ///
   /// Example usage:
   /// ```
+  /// # use staticvec::staticvec;
   /// assert_eq!(
   ///   staticvec![1, 2, 3].symmetric_difference(&staticvec![3, 4, 5]),
   ///   [1, 2, 4, 5]
@@ -1401,6 +1413,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   ///
   /// Example usage:
   /// ```
+  /// # use staticvec::staticvec;
   /// assert_eq!(
   ///   staticvec![4, 5, 6, 7].intersection(&staticvec![1, 2, 3, 7, 4]),
   ///   [4, 7],
@@ -1437,12 +1450,14 @@ impl<T, const N: usize> StaticVec<T, N> {
   ///
   /// Example usage:
   /// ```
+  /// # use staticvec::staticvec;
   /// assert_eq!(
   ///   staticvec![1, 2, 3].union(&staticvec![4, 2, 3, 4]),
   ///   [1, 2, 3, 4],
   /// );
   /// ```
   #[inline]
+  #[rustfmt::skip]
   pub fn union<const N2: usize>(&self, other: &StaticVec<T, N2>) -> StaticVec<T, { N + N2 }>
   where T: Clone + PartialEq {
     if self.length <= other.length {
@@ -1485,6 +1500,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   ///
   /// Example usage:
   /// ```
+  /// # use staticvec::{staticvec, StaticVec};
   /// const A: StaticVec<f64, 4> = staticvec![4.0, 5.0, 6.0, 7.0];
   /// const B: StaticVec<f64, 4> = staticvec![2.0, 3.0, 4.0, 5.0];
   /// assert_eq!(A.added(&B), [6.0, 8.0, 10.0, 12.0]);
@@ -1518,6 +1534,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   ///
   /// Example usage:
   /// ```
+  /// # use staticvec::{staticvec, StaticVec};
   /// const A: StaticVec<f64, 4> = staticvec![4.0, 5.0, 6.0, 7.0];
   /// const B: StaticVec<f64, 4> = staticvec![2.0, 3.0, 4.0, 5.0];
   /// assert_eq!(A.subtracted(&B), [2.0, 2.0, 2.0, 2.0]);
@@ -1551,6 +1568,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   ///
   /// Example usage:
   /// ```
+  /// # use staticvec::{staticvec, StaticVec};
   /// const A: StaticVec<f64, 4> = staticvec![4.0, 5.0, 6.0, 7.0];
   /// const B: StaticVec<f64, 4> = staticvec![2.0, 3.0, 4.0, 5.0];
   /// assert_eq!(A.multiplied(&B), [8.0, 15.0, 24.0, 35.0]);
@@ -1584,6 +1602,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   ///
   /// Example usage:
   /// ```
+  /// # use staticvec::{staticvec, StaticVec};
   /// const A: StaticVec<f64, 4> = staticvec![4.0, 5.0, 6.0, 7.0];
   /// const B: StaticVec<f64, 4> = staticvec![2.0, 3.0, 4.0, 5.0];
   /// assert_eq!(A.divided(&B), [2.0, 1.6666666666666667, 1.5, 1.4]);
